import javax.swing.JFrame;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import javax.swing.event.*;
import java.awt.*;

public class MainFrame extends JFrame {
	// the size of the down-sampled audio data block
	private static final int WAVE_DISPLAY_DATA_BLOCK_LEN = 40;
	
	// the control buttons on the bottom of the main window
	private JPanel pButton = null;
	private JButton bOnOff = null;
	private JButton bRecord = null;
	private JButton bPlayMusic = null;
	private JButton bMail = null;
	
	// the slider controls for the volume, frequency and echo settings 
	private JSlider slideMusicVolume = null;
	private JSlider slideVoiceVolume = null;
	private JSlider slideEcho = null;
	private JSlider slideFreq = null;
	
	// the default values of sliders
	private int musicVolume=5;
	private int voiceVolume=5;
	private int echoValue=0;
	private int freqValue=8;
	
	// the various panels on the main window
	private WavePanel pWaveDiagram = null;
	private JPanel pEffect = null;
	private JPanel pCenter = null;
	
	private boolean isOn = false;
	private boolean isRecording = false;
	
	// the distortion selection radio buttons
	private JRadioButton noDistortion = null;
	private JRadioButton patientDistortion = null;
	private JRadioButton walkyTalkyDistortion = null;
	private JRadioButton roboticDistortion = null;
	private JRadioButton gaussianDistortion = null;
	private JRadioButton lowerKeyDistortion = null;
	
	// the waveform panel update timer
	private Timer waveUpdateTime = null;
	private JFrame frame = null;
	
	// create the panel widgets for the button on the bottom of the frame
	private JPanel createButtonPanel() {
		pButton = new JPanel();
		
		Border raisedetched = BorderFactory.createEtchedBorder(EtchedBorder.RAISED);
		pButton.setBorder(raisedetched);
		
		// create the on/off button
		bOnOff = new JButton("On");
		bOnOff.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (isOn) {
					waveUpdateTime.stop();
					bOnOff.setText("On");
					ConsoleApp.send("2");
					isOn=false;
					bRecord.setEnabled(false);
					slideFreq.setEnabled(true);
				} else {
					waveUpdateTime.start();
					bOnOff.setText("Off");
					ConsoleApp.send("1");
					isOn=true;
					bRecord.setEnabled(true);
					slideFreq.setEnabled(false);
				}
			}
		});
		
		// create the start record /stop record button
		bRecord = new JButton("Start Record");
		bRecord.setEnabled(false);
		bRecord.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (isRecording) {
					isRecording=false;
					bRecord.setText("Start Record");
					ConsoleApp.send("4");
					bOnOff.setEnabled(true);
				} else {
					final JFileChooser fc = new JFileChooser();
//					In response to a button click:
					int returnVal = fc.showOpenDialog(frame);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						String fileName=fc.getSelectedFile().getAbsolutePath();
						isRecording=true;
						bRecord.setText("Stop Record");
						ConsoleApp.send("3");
						ConsoleApp.send(fileName);
						bOnOff.setEnabled(false);
					}
					
				}
			}
		});
		
		// create the play music button
		bPlayMusic = new JButton("Play Music");
		bPlayMusic.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JFileChooser fc = new JFileChooser();
//				In response to a button click:
				int returnVal = fc.showOpenDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String fileName=fc.getSelectedFile().getAbsolutePath();
					Runtime runtime=Runtime.getRuntime();
		
					try {
						Process proc=runtime.exec("c:\\PROGRA~1\\WINDOW~1\\wmplayer \""+fileName+"\"");
			
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			}
		});
		
		// create the mail delivery button
		bMail = new JButton("Mail");
		bMail.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
				final JFileChooser fc = new JFileChooser();
//				In response to a button click:
				int returnVal = fc.showOpenDialog(frame);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String fileName=fc.getSelectedFile().getAbsolutePath();
					String fileTextName=fc.getSelectedFile().getName();
					MailDialog dlg = new MailDialog(frame,fileName,fileTextName);
					dlg.show();
				}

			}
		});

		pButton.add(bOnOff);
		pButton.add(bRecord);
		pButton.add(bPlayMusic);
		pButton.add(bMail);
		return pButton;
	}

	// create the panel widgets for the effect settings on the right side of the frame
	private JPanel createEffectPanel() {
		
		pEffect = new JPanel();
		Border raisedetched = BorderFactory.createEtchedBorder(EtchedBorder.RAISED);
		pEffect.setBorder(raisedetched);
		GridLayout layout = new GridLayout(6,1);
		layout.setHgap(5);
		pEffect.setLayout(layout);

		// the music volume slider
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		slideMusicVolume = new JSlider(JSlider.HORIZONTAL,0,10,musicVolume);
		slideMusicVolume.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
				int musicVolume = (int)source.getValue();
				ConsoleApp.send("6 "+musicVolume);
		    }
		});
		panel.add(new JLabel("Music Volume "),BorderLayout.WEST);
		panel.add(slideMusicVolume,BorderLayout.CENTER);
		pEffect.add(panel);

		// the voice volume slider
		panel = new JPanel();
		panel.setLayout(new BorderLayout());
		slideVoiceVolume = new JSlider(JSlider.HORIZONTAL,0,10,voiceVolume);
		slideVoiceVolume.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
				int voiceVolume = (int)source.getValue();
				ConsoleApp.send("7 "+voiceVolume);
		    }
		});
		panel.add(new JLabel("Voice Volume "),BorderLayout.WEST);
		panel.add(slideVoiceVolume,BorderLayout.CENTER);
		pEffect.add(panel);

		panel = new JPanel();
		panel.setLayout(new BorderLayout());

		// the echo setting slider
		slideEcho = new JSlider(JSlider.HORIZONTAL,0,10,echoValue);
		slideEcho.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
				echoValue = (int)source.getValue();
				ConsoleApp.send("5 "+echoValue);
		    }
		});
		panel.add(new JLabel("Echo   "),BorderLayout.WEST);
		panel.add(slideEcho,BorderLayout.CENTER);
		pEffect.add(panel);

		// the frequency setting slider
		panel = new JPanel();
		panel.setLayout(new BorderLayout());
		slideFreq = new JSlider(JSlider.HORIZONTAL,1,8,freqValue);
		slideFreq.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider)e.getSource();
				freqValue = (int)source.getValue();
				ConsoleApp.send("8 "+freqValue);
		    }
		});
		panel.add(new JLabel("Frequency "),BorderLayout.WEST);
		panel.add(slideFreq,BorderLayout.CENTER);
		pEffect.add(panel);
		
		panel = new JPanel();

		// create the distortion configuration radio buttons
		DistortionEventHandler distortionHandler = new DistortionEventHandler();
		
		ButtonGroup distortionGroup = new ButtonGroup();
	    
	    noDistortion = new JRadioButton("No Distortion");
	    noDistortion.setActionCommand("0");
        noDistortion.setSelected(true);
	    noDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(noDistortion);

	    patientDistortion = new JRadioButton("Patient Distortion");
	    patientDistortion.setActionCommand("1");
	    patientDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(patientDistortion);
	    
	    walkyTalkyDistortion = new JRadioButton("Walky Talky Distortion");
	    walkyTalkyDistortion.setActionCommand("2");
	    walkyTalkyDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(walkyTalkyDistortion);

	    panel.add(noDistortion);
	    panel.add(patientDistortion);
	    panel.add(walkyTalkyDistortion);
	    pEffect.add(panel);

	    panel = new JPanel();
	    
	    roboticDistortion = new JRadioButton("Robotic Distortion");
	    roboticDistortion.setActionCommand("3");
	    roboticDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(roboticDistortion);

	    gaussianDistortion = new JRadioButton("Gaussian Distortion");
	    gaussianDistortion.setActionCommand("4");
	    gaussianDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(gaussianDistortion);

	    lowerKeyDistortion = new JRadioButton("Lower Key Distortion");
	    lowerKeyDistortion.setActionCommand("5");
	    lowerKeyDistortion.addActionListener(distortionHandler);
	    distortionGroup.add(lowerKeyDistortion);

	    panel.add(roboticDistortion);
	    panel.add(gaussianDistortion);
	    panel.add(lowerKeyDistortion);
	    
	    pEffect.add(panel);
		
		return pEffect; 
	}
	
	// create the main frame window
	public MainFrame() {
		super("Intelligent multimedia system");
		
		frame = this;
		
		// create the timer to update the wave display
		ActionListener taskPerformer = new ActionListener() {
			int[] buffer = new int[WAVE_DISPLAY_DATA_BLOCK_LEN];
		    public void actionPerformed(ActionEvent evt) {
		    	int idx = 0; 
		    	//System.out.println("Timer: "+ConsoleApp.available());
		    	while (ConsoleApp.available()>0 && idx < WAVE_DISPLAY_DATA_BLOCK_LEN) {
		    		buffer[idx]=Integer.parseInt(ConsoleApp.recv());
		    		idx++;
		    	}
		    	pWaveDiagram.addSample(buffer,idx);
		    }
		};
		waveUpdateTime=new Timer(100, taskPerformer);
		
		Container pMain = this.getContentPane();
	
		pCenter = new JPanel();
		pCenter.setLayout(new GridLayout());
		pCenter.add(createEffectPanel());
		BorderLayout layout = new BorderLayout();
		layout.setHgap(2);
		pMain.setLayout(layout);
		
		// create the wave display panel
		pWaveDiagram = new WavePanel();
		pMain.add(pWaveDiagram,BorderLayout.WEST);
		pMain.add(pCenter,BorderLayout.CENTER);
		pMain.add(createButtonPanel(),BorderLayout.SOUTH);
		
		// create the handler to release all the resource after the application exits
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				ConsoleApp.send("0");
				try {
					Thread.sleep(1000);
				} catch (Exception ex) {
					ex.printStackTrace();
				};
				
				System.exit(-1);
			}
			
		});
		
		this.setPreferredSize(new Dimension(1000,250));
		pack();
		
	}
}

// the class to handle the event of distortion selection
class DistortionEventHandler implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		ConsoleApp.send("9 "+e.getActionCommand());
	}
}